#!/bin/bash

MAX_FILES=${MAX_FILES:-20}
DIR=${DIR:-$1}
CLEANUP_WAIT=${CLEANUP_WAIT:-100}
CHECK_INTVL=${CHECK_INTVL:-5}

DURATION=${DURATION:-$2}
DURATION=${DURATION:-$((60*5))}
export PRT_INTVL=${PRT_INTVL:-10}

NUM_THREADS=${NUM_THREADS:-$3}
NUM_CPUS=$(grep -c '^processor' /proc/cpuinfo)
[ $NUM_CPUS -lt 1 ] && NUM_CPUS=1
NUM_THREADS=${NUM_THREADS:-$((NUM_CPUS * 2))}

PROGNAME=`basename $0`

if [ -z "$DIR" ]; then
	echo "usage: $PROGNAME {dir} [duration] [num_threads]" 1>&2
	exit 1
fi

if [ -e "$DIR" -a ! -d "$DIR" ]; then
	echo "$0: '$DIR' and '`basename $DIR`' are not directories"
	exit 1
fi

if [ ! -d "$DIR" ] && ! mkdir "$DIR"; then
	echo "$PROGNAME: FAIL: error creating '$DIR'"
	exit 1
fi

PROGLIST="file_create.sh dir_create.sh file_rm.sh file_rename.sh file_link.sh \
	  file_symlink.sh file_list.sh file_concat.sh"

racer_running()
{
	RUNNING=1
	if [ "$1" = "-r" ]; then # report if running
		SHOW_RUNNING=echo
		SHOW_STOPPED=:
		shift
	else
		SHOW_RUNNING=:
		SHOW_STOPPED=echo
	fi

	for PROG in $PROGLIST; do
		killall -0 -q $PROG && RUNNING=0 &&
			$SHOW_RUNNING "$PROGNAME: $PROG still running" || \
			$SHOW_STOPPED "$PROGNAME: $PROG no longer running"
	done
	
	return $RUNNING
}

racer_cleanup()
{
	echo "$PROGNAME: cleaning up"
	for PROG in $PROGLIST; do
		killall -q $PROG
	done
	trap 0
}

echo "$PROGNAME: run for ${DURATION}s with $NUM_THREADS threads. CTRL-C to exit"
trap racer_cleanup 2

cd `dirname $0`
for N in `seq 1 $NUM_THREADS`; do
	for PROG in $PROGLIST; do
		eval ./$PROG "$DIR" $MAX_FILES $N 2>/dev/null &
	done
done

RAN=0
while racer_running && [ $RAN -lt $DURATION ]; do
	sleep $CHECK_INTVL
	RAN=$((RAN + CHECK_INTVL))
	echo "$PROGNAME: running ${RAN}s at `date` (`date +%s`)"
done
racer_cleanup

# Check our to see whether our test DIR is still available.
df "$DIR"
RC=$?
if [ $RC -eq 0 ]; then
	echo "$PROGNAME: we survived for $RAN seconds."
else
	echo "$PROGNAME: FAIL: 'df $DIR' failed with $RC."
	exit $RC
fi

WAITED=0
while racer_running -r && [ $WAITED -lt $CLEANUP_WAIT ]; do
	sleep $CHECK_INTVL
	WAITED=$((WAITED + CHECK_INTVL))
done
if [ $WAITED -ge $CLEANUP_WAIT ]; then
	echo "$PROGNAME: FAIL: processes still running after ${CLEANUP_WAIT}s"
	exit 2
fi

rm -rf "$DIR"
RC=$?
if [ $RC -eq 0 ]; then
	echo "$PROGNAME: we cleaned up '$DIR' successfully."
else
	echo "$PROGNAME: FAIL: 'rm -rf $DIR' failed with $RC."
fi
exit $RC
